#!/usr/bin/perl -w

#
# Read report.conf from the location of this script.
#

my $confClusterName = "Cluster";
my $confNodes = "/var/lib/triumf_nodeinfo";
my $confWeb   = "/var/www/html/triumf_nodeinfo";

die "$confNodes does not exist: $!" if ! -d $confNodes;
die "$confWeb   does not exist: $!" if ! -d $confWeb;

mkdir $confWeb."/byOS",0777;
mkdir $confWeb."/byHost",0777;

# use bytes: If xml data files have garbage characters (i.e. from
# dmidecode), perl would try to interpret them as Unicode and would
# get confused. Tell it to not do Unicode. See man perlunicode.
#
use bytes;

my $html = "";

$html .= "<html>\n";
$html .= "<title>$confClusterName Configuration Page</title>\n";
$html .= "<h1>$confClusterName Configuration Page</h1>\n";

$html .= "<p>Last updated: ".localtime(time())."</p>\n";

$html .= "<p>Quick links:\n";
$html .= "<a href='#main'>main</a>\n";
$html .= "<a href='#nodeinfo'>nodeinfo</a>\n";
$html .= "<a href='#misc'>misc</a>\n";
$html .= "<a href='#pciconfig'>pciconfig</a>\n";
$html .= "<a href='#ideconfig'>ideconfig</a>\n";
$html .= "<a href='#scsiconfig'>scsiconfig</a>\n";
$html .= "<a href='#smartinfo'>smartinfo</a>\n";
$html .= "<a href='#diskscrub'>diskscrub</a>\n";
$html .= "<a href='#nisconfig'>nisconfig</a>\n";
$html .= "<a href='#netconfig'>netconfig</a>\n";
$html .= "<a href='#rpm'>rpm</a>\n";
#$html .= "<a href='#'></a>\n";
$html .= "</p>\n";

$html .= "<a name='main'></a>\n";
$html .= "<h2>Hardware and OS versions</h2>\n";

$html .= "<table border=1 width=90% align=center>\n";

$html .= "<tr>";
$html .= "<td>Hostname</td>";
$html .= "<td align=center>Uptime</td>";
$html .= "<td align=center>CPU</td>";
$html .= "<td align=center>RAM</td>";
$html .= "<td align=center>Swap</td>";
$html .= "<td align=center>RPMs</td>";
$html .= "<td align=center>OS and kernel</td>";
$html .= "<td align=center>Description</td>";
$html .= "</tr>\n";

system "rm -f $confWeb/byOS/*.txt";
system "rm -f $confWeb/byHost/*.txt";

my %nodedata;

my @descriptionheaders;

my $files = `ls -1 $confNodes/*.xml`;
foreach my $f (sort split(/\n/,$files))
{
  my $data = readFile($f);

  my $hostname = findXmlTag($data,"hostname");
  my $uptime0  = findXmlTag($data,"uptime");
  my $cpuinfo  = findXmlTag($data,"cpuinfo");
  my $meminfo  = findXmlTag($data,"meminfo");
  my $redhat   = findXmlTag($data,"redhat");
  my $linux    = findXmlTag($data,"linux");
  my $rpms     = findXmlTag($data,"rpms");
  my $etcMD5   = findXmlTag($data,"etcMD5sum");
  my $chkconfig = findXmlTag($data,"chkconfig");

  $hostname =~ s/\..+$// unless $hostname =~ /^\d+/;

  $nodedata{$hostname} = $data;

  $uptime = "";
  if (defined $uptime0)
    {
      $uptime0 =~ /up (.*?),/;
      $uptime = $1;
    }

  if (defined $chkconfig)
    {
      $chkconfig =~ s/.\:off//gs;
      $chkconfig =~ s/\:on//gs;
      $chkconfig =~ s/\t+/-/gs;
      $chkconfig =~ s/ +/-/gs;
      $chkconfig =~ s/-+/-/gs;
      $chkconfig =~ s/-$//gm;
      $chkconfig =~ s/^-/xinetd-/gm;
      #print "host $hostname, chkconfig: $chkconfig\n";

      $nodedata{$hostname}{"chkconfig"} = makeref(split(/\n/,$chkconfig));
    }

  $redhat =~ s/Red Hat Linux release/RH /;
  $redhat =~ s/Red Hat Enterprise Linux release/RHEL /;
  $redhat =~ s/Fedora Core release/FC /;
  $redhat =~ s/Scientific Linux SL Release/SL /;
  $redhat =~ s/Scientific Linux SL release/SL /;
  $redhat =~ s/Red Hat Enterprise Linux WS release/RHEL WS /;
  $redhat =~ s/ \(.*\)//;
  $redhat =~ s/  / /g;
  $redhat =~ s/ /-/g;

  $hosts{$redhat} .= "$hostname ";

  $nodedata{$hostname}{"os"} = $redhat;

  if (defined $rpms)
    {
      $rpms =~ s/^\n//s;
      $rpms =~ s/\n$//s;
      #$nodedata{$hostname}{"rpm1"} = readRpms($rpms);
      $nodedata{$hostname}{"rpms"} = makeref(split(/\n/,$rpms));
    }

  my $cpu = "";
  if (defined $cpuinfo)
    {
      my $count = 0;
      my $mhz = "?";
      foreach my $line (split(/\n/,$cpuinfo))
	{
	  $mhz = $2 if ($line =~ /MHz(.*):(.*)/);
	  $count ++ if ($line =~ /MHz(.*):(.*)/);
	}

      $cpu = int($mhz);
      $cpu = $count."x".int($mhz) if ($count>1);
    }

  my $totalram  = "";
  my $totalswap = "";
  if (defined $meminfo)
    {
      if ($meminfo =~ /Mem:\s+(\d+)/)
        {
          $totalram = int(int($1)/(1024*1024));
        }

      if ($meminfo =~ /MemTotal:\s+(\d+)/)
        {
          $totalram = int(int($1)/(1024));
        }

      if ($meminfo =~ /Swap:\s+(\d+)/)
        {
          $totalswap = int(int($1)/(1024*1024));
        }

      if ($meminfo =~ /SwapTotal:\s+(\d+)/)
        {
          $totalswap = int(int($1)/(1024));
        }
    }

  my $etcnodeinfo = findXmlTag($data,"etcnodeinfo");
  my $description = "";
  if (defined $etcnodeinfo)
    {
      if ($etcnodeinfo =~ /^(.*)\n/)
	{
	  $description = $1;
	}

      # discover all headers
      my @xxx = split(/\n/,$etcnodeinfo);
      # skip the first "description" line
      shift @xxx;
      foreach $hdr (@xxx)
	{
	  if ($hdr =~ /(.*?)\s*:\s*.*/)
	    {
	      my $hhh = $1;
	      if (! defined $headers{$hhh})
		{
		  push @descriptionheaders,$hhh;
		  $headers{$hhh} = 1;

                  #print "$hostname [$hhh] [$hdr]\n";
		}
	    }
	}
    }

  $description{$hostname} = $description;

  my $bgcolor = "";
  my $time = findXmlTag($data,"time");
  $bgcolor = "bgcolor=red" if (time() - $time > 24*60*60);

  $html .= "<tr $bgcolor>";
  $html .= "<td>$hostname</td>";
  $html .= "<td align=right>$uptime</td>";
  $html .= "<td align=right>$cpu</td>";
  $html .= "<td align=right>$totalram</td>";
  $html .= "<td align=right>$totalswap</td>";
  $html .= htmltag("td","align=right",0+@{$nodedata{$hostname}{"rpms"}});
  $html .= "<td>$redhat $linux</td>";
  $html .= "<td align=left>$description</td>";

  $html .= "</tr>\n";
}

$html .= "</table>\n";

my %rpmdata;
my %osdata;
my %chkconfigdata;

foreach my $h (sort keys %nodedata)
  {
    my $os = $nodedata{$h}{"os"};
    push @{$osdata{$os}{"hosts"}},$h;
    foreach my $rpm (@{$nodedata{$h}{"rpms"}})
      {
	push @{$rpmdata{$rpm}{"hosts"}},$h;
	push @{$osdata{$os}{"rpms"}},$rpm;
      }
    foreach my $chk (@{$nodedata{$h}{"chkconfig"}})
      {
	push @{$chkconfigdata{$chk}{"hosts"}},$h;
	push @{$osdata{$os}{"chkconfig"}},$chk;
      }
  }

$html .= "<a name='nodeinfo'></a>\n";
$html .= "<h2>Node information</h2>\n";

$html .= "<table border=1 width=90% align=center>\n";
$html .= "<tr>";
$html .= htmltag("td","align=center",htmltag("b","","Hostname"));
$html .= htmltag("td","align=center",htmltag("b","","Description"));
foreach my $hdr (@descriptionheaders)
  {
    $html .= htmltag("td","align=center",htmltag("b","","$hdr"));
  }
$html .= "</tr>\n";

foreach my $h (sort keys %nodedata)
  {
    my $data   = $nodedata{$h};
    my $etcnodeinfo = findXmlTag($data,"etcnodeinfo");

    $html .= "<tr>";
    $html .= htmltag("td","align=center",$h);
    $html .= htmltag("td","align=left",$description{$h});
    foreach my $hdr (@descriptionheaders)
      {
	my $value = "";
        if (defined $etcnodeinfo)
          {
            if ($etcnodeinfo =~ /^$hdr\s*:\s*(.*?)$/m)
              {
                $value = $1;
              }
          }
	$html .= htmltag("td","align=left",$value);
      }
    $html .= "</tr>\n";
  }

$html .= "</table>\n";

$html .= "<a name='misc'></a>\n";
$html .= "<h2>Misc configuration</h2>\n";

$html .= "<table border=1 width=90% align=center>\n";
$html .= "<tr>";
$html .= htmltag("td","align=center",htmltag("b","","Hostname"));
$html .= htmltag("td","align=center",htmltag("b","","lpr"));
$html .= htmltag("td","align=center",htmltag("b","","ntp"));
$html .= htmltag("td","align=center",htmltag("b","","root .forward"));
$html .= htmltag("td","align=center",htmltag("b","","tcp wrappers"));
$html .= htmltag("td","align=center",htmltag("b","","mail"));
$html .= "</tr>\n";

foreach my $h (sort keys %nodedata)
  {
    my $data   = $nodedata{$h};

    $html .= "<tr>";
    $html .= htmltag("td","align=center",$h);

    if (my $printcap = findXmlTag($data,"printcap"))
      {
	if ($printcap =~ /cupsd/)
	  {
	    $html .= htmltag("td","align=center","cups");
	  }
	else
	  {
	    my @lp = split(/\n/,$printcap);
	    my $out = "";
	    foreach my $printer (@lp)
	      {
		next if $printer =~ /^#/;
		$printer =~ s/:.*$//;
		$out .= " ".$printer;
	      }
	    $html .= htmltag("td","",$out);
	  }
      }
    else
      {
	$html .= htmltag("td","bgcolor=red align=center","none");
      }

    if (my $ntp = findXmlTag($data,"ntp"))
      {
	#$html .= htmltag("td","","$ntp");
	my $x = "";
	foreach my $d (split(/\n/,$ntp))
	  {
	    next if $d =~ /^local/;
	    $d =~ /^(.*):/;
	    my $host = $1;
	    $host =~ s/\.(.*)$//;
	    $x .= "$host ";
	    last;
	  }
	$html .= htmltag("td","align=center","$x");
      }
    elsif (my $ntpstat = findXmlTag($data,"ntpstat"))
      {
        if ($ntpstat =~ /synchronised to NTP server \((.*)\)/)
          {
            $html .= htmltag("td","align=center",$1);
          }
        else
          {
            $html .= htmltag("td","bgcolor=red align=center","none");
          }
      }
    else
      {
	$html .= htmltag("td","bgcolor=red align=center","none");
      }

    if (my $ret = findXmlTag($data,"rootforward"))
      {
	$html .= htmltag("td","align=center","yes");
      }
    else
      {
	$html .= htmltag("td","align=center","?");
      }

    if (my $ret = findXmlTag($data,"tcpwrappers"))
      {
        my @hhh;
        push @hhh,"hosts.allow" if $ret =~ /hosts.allow/;
        push @hhh,"hosts.deny"  if $ret =~ /hosts.deny/;

        $html .= htmltag("td","align=center",join(" ",@hhh));
      }
    else
      {
	$html .= htmltag("td","align=center","?");
      }

    if (my $mail = findXmlTag($data,"mailspool"))
      {
	#$html .= htmltag("td","","$mail");
	my $x = "";
	foreach my $d (split(/\n/,$mail))
	  {
	    next if $d =~ /^total/;
	    my @d = split(/ +/,$d);
	    my $name = pop @d;
	    my $size = $d[4];
	    $x .= "$name($size) ";
	  }
	$html .= htmltag("td","","$x");
      }
    else
      {
	$html .= htmltag("td","bgcolor=red align=center","none");
      }

    $html .= "</tr>\n";
  }

$html .= "</table>\n";

{
  $html .= "<a name='pciconfig'></a>\n";
  $html .= "<h2>PCI configuration</h2>\n";

  $html .= "<table border=1 width=90% align=center>\n";
  $html .= "<tr>";
  $html .= htmltag("td","align=center",htmltag("b","","Hostname"));
  $html .= htmltag("td","align=center",htmltag("b","","lspci"));
  $html .= "</tr>\n";
  
  foreach my $h (sort keys %nodedata)
    {
      my $data = $nodedata{$h};
      my $conf = findXmlTag($data,"lspci");
      
      if (defined $conf)
        {
          $conf =~ s/\n/\<br\>/mg;
          $conf =~ s/ /\&nbsp\;/mg;
          $html .= "<tr>";
          $html .= htmltag("td","align=center",$h);
          $html .= htmltag("td","align=left",$conf);
          $html .= "</tr>\n";
        }
    }
  
  $html .= "</table>\n";
}

$html .= "<a name='ideconfig'></a>\n";
$html .= "<h2>IDE configuration</h2>\n";

$html .= "<table border=1 width=90% align=center>\n";
$html .= "<tr>";
$html .= htmltag("td","align=center",htmltag("b","","Hostname"));
$html .= htmltag("td","align=center",htmltag("b","","Slot"));
$html .= htmltag("td","align=center",htmltag("b","","Capacity"));
$html .= htmltag("td","align=center",htmltag("b","","Cache"));
$html .= htmltag("td","align=left",htmltag("b","","Identity"));
$html .= "</tr>\n";

foreach my $h (sort keys %nodedata)
  {
    my $data   = $nodedata{$h};
    my $ideconf = findXmlTag($data,"ide");

    if (defined $ideconf)
      {
	my ($slots, $caps, $caches, $models);
	foreach my $dev (sort split(/\n/,$ideconf))
	  {
	    my ($slot,$cap,$cache,$model) = split(/\:/,$dev);
	    my $capGB = int($cap/(2*1000*1000));
	    # some CDROMs report non-zero capacity
	    if (($capGB == 1073)||($capGB == 0))
	      {
		$capGB = "(removable)";
	      }
	    else
	      {
		$capGB .= " GBytes" if ($capGB > 0);
	      }
	    $cache = "" if ($cache eq "none");

	    $slots .= "$slot<br>";
	    $caps  .= "$capGB<br>";
	    $caches.= "$cache<br>";
	    $models.= "$model<br>";
	  }

	$html .= "<tr>";
	$html .= htmltag("td","align=center",$h);
        if (defined $slots)
          {
            $html .= htmltag("td","align=center",$slots);
            $html .= htmltag("td","align=right",$caps);
            $html .= htmltag("td","align=right",$caches);
            $html .= htmltag("td","align=left",$models);
          }
	$html .= "</tr>\n";
      }
  }

$html .= "</table>\n";

$html .= "<a name='scsiconfig'></a>\n";
$html .= "<h2>SCSI configuration</h2>\n";

$html .= "<table border=1 width=90% align=center>\n";
$html .= "<tr>";
$html .= htmltag("td","align=center",htmltag("b","","Hostname"));
$html .= htmltag("td","align=center",htmltag("b","","/proc/scsi/scsi"));
$html .= "</tr>\n";

foreach my $h (sort keys %nodedata)
  {
    my $data = $nodedata{$h};
    my $conf = findXmlTag($data,"scsi");

    if (defined $conf)
      {
	$conf =~ s/\n/\<br\>/mg;
	$conf =~ s/ /\&nbsp\;/mg;
	$html .= "<tr>";
	$html .= htmltag("td","align=center",$h);
	$html .= htmltag("td","align=left",$conf);
	$html .= "</tr>\n";
      }
  }

$html .= "</table>\n";

{
  $html .= "<a name='smartinfo'></a>\n";
  $html .= "<h2>SMART disk diagnostics</h2>\n";

  my %smart;

  foreach my $h (sort keys %nodedata)
    {
      my $data = findXmlTag($nodedata{$h},"smart");
      next if ! $data;
      foreach my $d (split(/\<data/,$data))
        {
          if ($d =~ /dev\=\"(.*)\"\>/)
            {
              my $disk = $h.":".$1;

              #print "$h $disk $d\n";

              next if $d =~ /Device Read Identity Failed/;

              my $disktype = "unknown";

              if ($d =~ /Device: (\w+)\W(.*)\s+Supports/)
                {
                  $disktype  = uc($1);
                  my $diskmodel = $2;

                  if (lc($disktype) =~ /^st/)
                    {
                      $diskmodel = $disktype;
                      $disktype = "Seagate";
                    }

                  if (lc($disktype) =~ /^ic/)
                    {
                      $diskmodel = $disktype;
                      $disktype = "IBM";
                    }

                  $smart{$disktype}{"attr"}{"00model"} = "model";
                  $smart{$disktype}{$disk}{"00model"} = $diskmodel;

                  #print "A [$disk] [$disktype] [$diskmodel]\n";
                }
              elsif ($d =~ /^Device Model:\s+(\w+)\s+(.*)$/m)
                {
                  $disktype  = uc($1);
                  my $diskmodel = $2;

                  if (lc($disktype) =~ /^st/)
                    {
                      $diskmodel = $disktype;
                      $disktype = "Seagate";
                    }

                  if (lc($disktype) =~ /^ic/)
                    {
                      $diskmodel = $disktype;
                      $disktype = "IBM";
                    }

                  $smart{$disktype}{"attr"}{"00model"} = "model";
                  $smart{$disktype}{$disk}{"00model"} = $diskmodel;

                  #print "B [$disk] [$disktype] [$diskmodel]\n";
                }
              elsif ($d =~ /^Device Model:\s+(\w+)\W+(.*)$/m)
                {
                  $disktype  = uc($1);
                  my $diskmodel = $2;

                  if (lc($disktype) =~ /^st/)
                    {
                      $diskmodel = $disktype;
                      $disktype = "Seagate";
                    }

                  if (lc($disktype) =~ /^ic/)
                    {
                      $diskmodel = $disktype;
                      $disktype = "IBM";
                    }

                  $smart{$disktype}{"attr"}{"00model"} = "model";
                  $smart{$disktype}{$disk}{"00model"} = $diskmodel;

                  #print "C [$disk] [$disktype] [$diskmodel]\n";
                }

              if ($d =~ /^Serial Number:\s+(.*)$/m)
                {
                  $smart{$disktype}{"attr"}{"00serno"} = "serno";
                  $smart{$disktype}{$disk}{"00serno"} = $1;
                }

              if ($d =~ /^Firmware Version:\s+(.*)$/m)
                {
                  $smart{$disktype}{"attr"}{"01firmware"} = "firmware";
                  $smart{$disktype}{$disk}{"01firmware"} = $1;
                }

              foreach my $a (split(/\n/,$d))
                {
                  if ($a =~ /\((..\d)\)(.+?)\s+0x\w+\s+\d+\s+\d+\s+\d+\s+(\d+)/)
                    {
                      # parse RHL8.0-9 smartctl data

                      my $attr = int($1);
                      my $name = $2;
                      my $val  = int($3);

                      next if $attr <= 0;
                      next if $val == 0;

                      #print "$disk $attr $name $val\n";

                      $smart{$disktype}{"attr"}{$attr} = $name;
                      $smart{$disktype}{$disk}{$attr} = $val;
                    }
                  elsif ($a =~ /ATA Error Count:\s+(\d+)/)
                    {
                      # parse RHL8.0-9 smartctl data

                      my $attr = "02ataerrors";
                      my $name = "ATA Error Count";
                      my $val  = int($1);

                      next if $val == 0;

                      #print "$disk $attr $name $val\n";

                      $smart{$disktype}{"attr"}{$attr} = $name;
                      $smart{$disktype}{$disk}{$attr} = $val;
                    }
                  elsif ($a =~ /(..\d)\s(\S+)\s+0x\w+\s+\d+\s+\d+\s+\d+\s+\S+\s+\S+\s+(\d+)/)
                    {
                      # parse Fedora-1 smartctl version 5.1-11 data

                      my $attr = int($1);
                      my $name = $2;
                      my $val  = int($3);

                      next if $attr <= 0;
                      next if $val == 0;

                      #print "$disk $attr $name $val\n";

                      $smart{$disktype}{"attr"}{$attr} = $name;
                      $smart{$disktype}{$disk}{$attr} = $val;
                    }
                  elsif ($a =~ /(..\d)\s(\S+)\s+0x\w+\s+\d+\s+\d+\s+\d+\s+\S+\s+\S+\s+\S+\s+(\d+)/)
                    {
                      # parse smartctl version 5.26 data

                      my $attr = int($1);
                      my $name = $2;
                      my $val  = int($3);

                      next if $attr <= 0;
                      next if $val == 0;

                      #print "$disk $attr $name $val\n";

                      $smart{$disktype}{"attr"}{$attr} = $name;
                      $smart{$disktype}{$disk}{$attr} = $val;
                    }
                }
            }
        }
    }

  foreach my $disktype (sort keys %smart)
    {
      #delete $smart{$disktype}{"attr"}{"00model"};
      #delete $smart{$disktype}{"attr"}{"00serno"};
      delete $smart{$disktype}{"attr"}{"01firmware"};
      #delete $smart{$disktype}{"attr"}{197}; # current pending sector
      
      $html .= "<h3>SMART disk diagnostics for $disktype disks</h3>\n";

      $html .= "<table border=1 width=90% align=center>\n";

      $html .= "<tr>";
      $html .= htmltag("td","align=center",htmltag("b","","Disk"));

      my @attr = sort keys %{$smart{$disktype}{"attr"}};
      foreach my $attr (@attr)
        {
          my $attrname = $smart{$disktype}{"attr"}{$attr};
          $attrname = $attr if ! $attrname;
          $attrname = lc($attrname);
          $attrname =~ s/\_/ /g;
          $attrname = $attr if ($attrname eq "unknown attribute");
          $attrname = "multi zone error rate" if ($disktype eq "WDC") && ($attrname eq "200");
          $attrname = "soft read error rate" if ($attrname eq "201");
          $attrname = "ECC errors" if ($disktype eq "MAXTOR") && ($attrname eq "203");
          $attrname =~ s/temperature/temp<br>era<br>ture/;
          $attrname =~ s/reallocated/re<br>allo<br>cated/;
          $attrname =~ s/power-off/power<br>off/;
          $attrname =~ s/uncorrectable/un<br>correct<br>able/;
          $attrname =~ s/preformance/per<br>for<br>mance/;
          $attrname =~ s/performance/per<br>for<br>mance/;
          $attrname =~ s/hardware/hard<br>ware/;
          $attrname =~ s/recovered/reco<br>vered/;
          $attrname =~ s/calibration/calib<br>ration/;
          #$attrname = $attr."<br>".$attrname;
          $html .= htmltag("td","align=center",htmltag("b","",$attrname));
        }
      $html .= "</tr>\n";

      foreach my $disk (sort keys %{$smart{$disktype}})
        {
          next if $disk eq "attr";

	  my $line = "";
	  my $diskcode = 0;

          foreach my $attr (@attr)
            {
              my $attrval = $smart{$disktype}{$disk}{$attr};
              $attrval = "" if ! $attrval;
              $attrval =~ s/\W/ /g;

	      my $code = 0;
	      $code |= 1 if (($attr eq "196")&&($attrval ne "")); # reallocated event count
	      $code |= 1 if (($attr eq "5")&&($attrval ne "")); # reallocated sector count
	      $code |= 2 if (($attr eq "197")&&($attrval ne "")); # current pending sector

	      $diskcode |= $code;

	      my $bgcolour = "";
	      $bgcolour = " bgcolor=yellow" if $code > 0;
	      $bgcolour = " bgcolor=red"    if $code > 1;

              $line .= htmltag("td","align=right".$bgcolour,$attrval);
            }

	  my $diskcolour = "";
	  $diskcolour = " bgcolor=yellow" if $diskcode > 0;
	  $diskcolour = " bgcolor=red"    if $diskcode > 1;

          $html .= "<tr>";
	  $html .= htmltag("td","align=center".$diskcolour,$disk);
	  $html .= $line;
          $html .= "</tr>\n";
        }

      $html .= "</table>\n";
    }
}

$html .= "<a name='diskscrub'></a>\n";
$html .= "<h2>Diskscrub results</h2>\n";

$html .= "<table border=1 width=90% align=center>\n";
$html .= "<tr>";
$html .= htmltag("td","align=center",htmltag("b","","Disk"));
$html .= htmltag("td","align=center",htmltag("b","","Size, GiB"));
$html .= htmltag("td","align=center",htmltag("b","","Slow reads"));
$html .= htmltag("td","align=center",htmltag("b","","Read errors"));
$html .= htmltag("td","align=center",htmltag("b","","Slowest read, sec"));
$html .= htmltag("td","align=center",htmltag("b","","AveRate, MiB/sec"));
$html .= htmltag("td","align=center",htmltag("b","","MinRate, MiB/sec"));
$html .= htmltag("td","align=center",htmltag("b","","MaxRate, MiB/sec"));
$html .= "</tr>\n";

foreach my $h (sort keys %nodedata)
  {
    my $data    = $nodedata{$h};
    my $scrub   = findXmlTag($data,"diskscrub");

    next unless $scrub;

    #print "$h: $scrub\n";

    my $on = 0;
    foreach my $line (split(/\n/,$scrub))
      {
        $on++ if $line =~ /======================/;
        next unless $on == 1;
        next unless $line =~ /\/dev/;

	#print "scrub: $line\n";

        my ($blank,$dev,$size,$slow,$short,$errors,$minrate,$maxrate,$slowread);
	my $averate = "";

	if ($line =~ /^\s+/)
	  {
	    ($blank,$dev,$size,$slow,$short,$errors,$minrate,$maxrate,$slowread) = split(/\s+/,$line);
	  }
	else
	  {
	    ($dev,$size,$slow,$short,$errors,$minrate,$maxrate,$averate,$slowread) = split(/\s+/,$line);
	  }

        my $disk = "$h:$dev";

        $errors = $errors + $short;

        $slow   = "" unless $slow != 0;
        $errors = "" unless $errors != 0;

        my $estatus = 0;
        my $sstatus = 0;
        my $rstatus = 0;

        $estatus |= 2 if $errors;
	if (defined $slowread)
	  {
	    $sstatus |= 1 if $slowread >  5.0;
	    $sstatus |= 2 if $slowread > 20.0;
	  }
	if (defined $maxrate)
	  {
	    $rstatus |= 1 if $maxrate  < 10.0;
	  }

        $html .= "<tr>";
        $html .= htmltag("td","align=center ".statusColour($estatus|$sstatus|$rstatus),$disk);
        $html .= htmltag("td","align=right",$size);
        $html .= htmltag("td","align=right",$slow);
        $html .= htmltag("td","align=right ".statusColour($estatus),$errors);
        $html .= htmltag("td","align=right ".statusColour($sstatus),$slowread);
        $html .= htmltag("td","align=right",$averate);
        $html .= htmltag("td","align=right",$minrate);
        $html .= htmltag("td","align=right ".statusColour($rstatus),$maxrate);
        $html .= "</tr>\n";
      }
  }

$html .= "</table>\n";

$html .= "<a name='nisconfig'></a>\n";
$html .= "<h2>NIS configuration</h2>\n";

$html .= "<table border=1 width=90% align=center>\n";
$html .= "<tr>";
$html .= htmltag("td","align=center",htmltag("b","","Hostname"));
$html .= htmltag("td","align=center",htmltag("b","","Domain"));
$html .= htmltag("td","align=center",htmltag("b","","ypserv"));
$html .= htmltag("td","align=center",htmltag("b","","ypwhich"));
$html .= htmltag("td","align=center",htmltag("b","","/etc/yp.conf"));
$html .= htmltag("td","align=center",htmltag("b","","/var/yp/securenets"));
$html .= "</tr>\n";

foreach my $h (sort keys %nodedata)
  {
    my $data    = $nodedata{$h};
    my $domain  = findXmlTag($data,"domainname");
    my $ypconf  = findXmlTag($data,"etcypconf");
    my $ypwhich = findXmlTag($data,"ypwhich");
    my $ypserv  = findXmlTag($data,"ypserv");

    my $yp;

    if (defined $ypconf)
      {
	$ypconf =~ /domain\s+(\S+)/;
	my $ypdomain = $1;

	if (defined $ypdomain)
	  {
	    my $broadcast = "";
	    $broadcast = " broadcast" if $ypconf =~ /broadcast/;

	    my $server = "";
	    $server = " server $1" if $ypconf =~ /server\s+(\S+)/;

	    $yp = "$server$broadcast";
	  }
      }

    $html .= "<tr>";
    $html .= htmltag("td","align=center",$h);
    if ($domain || $ypwhich || $yp)
      {
        $domain = "" if ! $domain;

        $html .= htmltag("td","align=center",$domain);
        if (!defined $ypserv)
          {
            $html .= htmltag("td","align=center","");
          }
        elsif ($ypserv =~ /\d+/)
          {
            $html .= htmltag("td","align=center",$ypserv);
          }
        else
          {
            $html .= htmltag("td","align=center bgcolor=red","missing");
          }
        $html .= htmltag("td","align=center",$ypwhich) if defined $ypwhich;
        $html .= htmltag("td","align=center bgcolor=red","not bound") if ! defined $ypwhich;
        $html .= htmltag("td","align=center",$yp)      if defined $yp;
        $html .= htmltag("td","align=center bgcolor=red","error")  if ! defined $yp;

        if (my $ret = findXmlTag($data,"securenets"))
          {
            $html .= htmltag("td","align=center","yes");
          }
        else
          {
            $html .= htmltag("td","align=center","-");
          }
      }
    $html .= "</tr>\n";
  }

$html .= "</table>\n";

$html .= "<a name='netconfig'></a>\n";
$html .= "<h2>Network configuration</h2>\n";

$html .= "<table border=1 width=90% align=center>\n";
$html .= "<tr>";
$html .= htmltag("td","",htmltag("b","","Hostname"));
$html .= htmltag("td","",htmltag("b","","Link"));
$html .= htmltag("td","",htmltag("b","","DHCP"));
$html .= htmltag("td","",htmltag("b","","ifconfig"));
$html .= htmltag("td","",htmltag("b","","DNS"));
$html .= "</tr>\n";

foreach my $h (sort keys %nodedata)
  {
    my $data = $nodedata{$h};
    my $link = findXmlTag($data,"miitool");
    $link = "(unknown)" if ! defined $link;

    my $xlink = "";
    foreach my $linkk (sort split(/\n/,$link))
      {
	#$xlink .= "100-FD" if $link eq "eth0: negotiated 100baseTx-FD, link ok";
	#$link = "100-FD-FC" if $link eq "eth0: negotiated 100baseTx-FD flow-control, link ok";

	$linkk = "$1: no link" if $linkk =~ /(.*):.*no link/;
	$linkk =~ s/negotiated//;
	$linkk =~ s/baseTx-/ /;
	$linkk =~ s/link ok//;
	$linkk =~ s/flow-control/FC/;
	$linkk =~ s/full duplex/FD/;
	$linkk =~ s/\,/ /g;
	$linkk =~ s/\s+/&nbsp;/g;

	$xlink .= "$linkk<br>";
      }

    my $eth0 = findXmlTag($data,"ifcfgeth0");
    my $dhcp;
    if (defined $eth0)
      {
	$dhcp = "dhcp" if $eth0 =~ /BOOTPROTO=.*dhcp.*/;
	$dhcp = "static" if $eth0 =~ /IPADDR=/;
      }

    my $resolv = findXmlTag($data,"etcresolvconf");
    my $dns;
    if (defined $resolv)
      {
	$dns = "";
	foreach my $line (split (/\n/,$resolv))
	  {
	    next if ! ($line =~ /^nameserver\s+(\S+)/);
	    $dns .= " ".$1;
	  }
      }

    my $ifconfig = "";
    my $xifconfig = findXmlTag($data,"ifconfig");
    if (defined $xifconfig)
      {
        my @x = split(/\n\n/,$xifconfig);

        foreach my $x (sort @x)
          {
            my ($name) = split(/\s+/,$x);

            next if $name =~ /^lo/;

            my $ip = "";
            my $mac = "";

            $ip  = $1 if ($x =~ /inet addr:(.*?)\s/);
            $mac = $1 if ($x =~ /HWaddr (.*?)\s/);

            $ifconfig .= "$name: $mac $ip<br>";
          }
      }

    $html .= "<tr>";
    $html .= htmltag("td","",$h);
    $html .= htmltag("td","",$xlink);
    $html .= htmltag("td","",$dhcp) if defined $dhcp;
    $html .= htmltag("td","bgcolor=red","error")  if ! defined $dhcp;
    $html .= htmltag("td","",$ifconfig);
    $html .= htmltag("td","",$dns) if defined $dns;
    $html .= htmltag("td","bgcolor=red","error")  if ! defined $dns;
    $html .= "</tr>\n";
  }

$html .= "</table>\n";

my %defdata;

#foreach my $os (sort keys %hosts)
#  {
#    my $reqrpmlist = readFile("$confNodes/../$os-rpms.txt");
#    $defdata{$os}{"rpm1"}      = readRpms($reqrpmlist);
#    $defdata{$os}{"rpms"}      = makeref(split(/\n/,$reqrpmlist));
#    $defdata{$os}{"chkconfig"} = makeref(split(/\n/,readFile("$confNodes/../$os-chkconfig.txt")));
#    #print "def data: ".join("X",@{$defdata{$os}{"rpms"}})."\n";
#  }

my %zchkconfig_hosts;
my %zchkconfig_chk;
my %zchkconfig_all;
my %zchkconfig_common;

my %zrpms_hosts;
my %zrpms_rpm;
my %zrpms_all;
my %zrpms_common;

foreach my $os (sort keys %osdata)
  {
    my $xallhosts = $osdata{$os}{"hosts"};
    foreach my $h (sort @{$xallhosts})
      {
        foreach my $rpm (@{$nodedata{$h}{"rpms"}})
          {
            $zrpms_hosts{$rpm}{$h} = 1;
            $zrpms_rpm{$h}{$rpm}   = 1;
            $zrpms_all{$os}{$rpm}  = 1;
          }

        foreach my $chk (@{$nodedata{$h}{"chkconfig"}})
          {
            $zchkconfig_hosts{$chk}{$h} = 1;
            $zchkconfig_chk{$h}{$chk}   = 1;
            $zchkconfig_all{$os}{$chk}  = 1;
          }
      }
  }

$html .= "<a name='rpm'></a>\n";
$html .= htmltag("h2","","RPMs and chkconfigs by OS")."\n";
$html .= "<table border=1 width=90% align=center>\n";

$html .= "<tr>";
$html .= htmltag("th","rowspan=2","OS");
$html .= htmltag("th","colspan=4","RPMs");
$html .= htmltag("th","colspan=4","chkconfig");
$html .= "</tr>\n";

$html .= "<tr>";
$html .= htmltag("th","","all");
$html .= htmltag("th","","common");
$html .= htmltag("th","","extra");
$html .= htmltag("th","","missing");
$html .= htmltag("th","","enabled");
$html .= htmltag("th","","common");
$html .= htmltag("th","","extra");
$html .= htmltag("th","","missing");
$html .= "</tr>\n";

foreach my $os (sort keys %hosts)
  {
    $html .= "<tr>";
    $html .= htmltag("td","",$os);

    my @rpm_all;
    my @rpm_common;
    my @rpm_extra;
    my @rpm_missing;

    my @zallhosts = sort split(/ /,$hosts{$os});

    foreach my $rpm (sort keys %{$zrpms_all{$os}})
      {
        my $hosts = join(" ",sort keys %{$zrpms_hosts{$rpm}});
        push @rpm_all,"$rpm: ".$hosts;

        my $flag = 1;
        my @missing_hosts;
        foreach my $h (@zallhosts)
          {
            if (! defined $zrpms_hosts{$rpm}{$h})
              {
                $flag = 0;
                push @missing_hosts,$h;
                $zrpms_missing{$h}{$rpm} = 1;
              }
          }

        if ($flag)
          {
            push @rpm_common,"$rpm: ".$hosts;
            $zrpms_common{$os}{$rpm} = 1;
          }
        else
          {
            push @rpm_extra,"$rpm: ".$hosts;
            push @rpm_missing,"$rpm: ".join(" ",@missing_hosts);
          }

        #print "$rpm: [$hosts] [$zallhosts]\n";
      }

    $qrpms_common_label{$os} = writeFile("byOS/".$os."_rpms_common.txt",join("\n",@rpm_common),scalar @rpm_common);

    $html .= htmltag("td","align=right",writeFile("byOS/".$os."_rpms_all.txt",join("\n",@rpm_all),scalar @rpm_all));
    $html .= htmltag("td","align=right",$qrpms_common_label{$os});
    $html .= htmltag("td","align=right",writeFile("byOS/".$os."_rpms_extra.txt",join("\n",@rpm_extra),scalar @rpm_extra));
    $html .= htmltag("td","align=right",writeFile("byOS/".$os."_rpms_missing.txt",join("\n",@rpm_missing),scalar @rpm_missing));

    my @chk_all;
    my @chk_common;
    my @chk_extra;
    my @chk_missing;

    foreach my $chk (sort keys %{$zchkconfig_all{$os}})
      {
        my $hosts = join(" ",sort keys %{$zchkconfig_hosts{$chk}});
        push @chk_all,"$chk: ".$hosts;

        my $flag = 1;
        my @missing_hosts;
        foreach my $h (@zallhosts)
          {
            if (! defined $zchkconfig_hosts{$chk}{$h})
              {
                $flag = 0;
                push @missing_hosts,$h;
                $zchkconfig_missing{$h}{$chk} = 1;
              }
          }

        if ($flag)
          {
            push @chk_common,"$chk: ".$hosts;
            $zchkconfig_common{$os}{$chk} = 1;
          }
        else
          {
            push @chk_extra,"$chk: ".$hosts;

            push @chk_missing,"$chk: ".join(" ",@missing_hosts);
          }

        #print "$chk: [$hosts] [$zallhosts]\n";
      }

    $qchkconfig_common_label{$os} = writeFile("byOS/".$os."_chkconfig_common.txt",join("\n",@chk_common),scalar @chk_common);

    $html .= htmltag("td","align=right",writeFile("byOS/".$os."_chkconfig_all.txt",join("\n",@chk_all),scalar @chk_all));
    $html .= htmltag("td","align=right",$qchkconfig_common_label{$os});
    $html .= htmltag("td","align=right",writeFile("byOS/".$os."_chkconfig_extra",join("\n",@chk_extra),scalar @chk_extra));
    $html .= htmltag("td","align=right",writeFile("byOS/".$os."_chkconfig_missing",join("\n",@chk_missing),scalar @chk_missing));

    $html .= "</tr>\n";
  }

$html .= "</table>\n";

$html .= htmltag("h2","","RPM and chkconfig configuration")."\n";

$html .= "<table border=1 width=90% align=center>\n";

$html .= "<tr>";
$html .= htmltag("th","rowspan=2","Hostname");
$html .= htmltag("th","rowspan=2","OS");
$html .= htmltag("th","colspan=6","RPMs");
$html .= htmltag("th","colspan=4","chkconfig");
$html .= "</tr>\n";
$html .= "<tr>";
$html .= htmltag("th","","inst");
$html .= htmltag("th","","common");
$html .= htmltag("th","","extra");
$html .= htmltag("th","","missing");
$html .= htmltag("th","",".rpmnew");
$html .= htmltag("th","",".rpmsave");
$html .= htmltag("th","","enabled");
$html .= htmltag("th","","common");
$html .= htmltag("th","","extra");
$html .= htmltag("th","","missing");
$html .= "</tr>\n";

foreach my $os (sort keys %osdata)
  {
    my $xallhosts = $osdata{$os}{"hosts"};
    foreach my $h (sort @{$xallhosts})
      {
	$html .= "<tr>";
	$html .= htmltag("td","",$h);
	$html .= htmltag("td","",$os);

        my @rpm = sort keys %{$zrpms_rpm{$h}};
        my @rpmextra;
        my @rpmmissing = sort keys %{$zrpms_missing{$h}};

        foreach my $rpm (@rpm)
          {
            push @rpmextra, $rpm if ! defined $zrpms_common{$os}{$rpm};
          }

	$html .= htmltag("td","align=right",writeFile("byHost/".$h."_rpms_inst.txt",join("\n",@rpm),scalar @rpm));
	$html .= htmltag("td","align=right",$qrpms_common_label{$os});
	$html .= htmltag("td","align=right",writeFile("byHost/".$h."_rpms_extra.txt",join("\n",@rpmextra),scalar @rpmextra));
	$html .= htmltag("td","align=right",writeFile("byHost/".$h."_rpms_missing.txt",join("\n",@rpmmissing),scalar @rpmmissing));

	my $rpmnew  = findXmlTag($nodedata{$h},"rpmnew");
	my $rpmsave = findXmlTag($nodedata{$h},"rpmsave");
	$rpmnew  = "" if ! defined $rpmnew;
	$rpmsave = "" if ! defined $rpmsave;
	my @rpmnew  = split(/\n/,$rpmnew);
	my @rpmsave = split(/\n/,$rpmsave);

	$html .= htmltag("td","align=right",writeFile("byHost/".$h."_rpmnew.txt",join("\n",@rpmnew),scalar @rpmnew));
	$html .= htmltag("td","align=right",writeFile("byHost/".$h."_rpmsave.txt",join("\n",@rpmsave),scalar @rpmsave));
        
        my @chk = sort keys %{$zchkconfig_chk{$h}};
        my @chkextra;
        my @chkmissing = sort keys %{$zchkconfig_missing{$h}};

        foreach my $chk (@chk)
          {
            push @chkextra, $chk if ! defined $zchkconfig_common{$os}{$chk};
          }

	$html .= htmltag("td","align=right",writeFile("byHost/".$h."_chkconfig_inst.txt",join("\n",@chk),scalar @chk));
	$html .= htmltag("td","align=right",$qchkconfig_common_label{$os});
	$html .= htmltag("td","align=right",writeFile("byHost/".$h."_chkconfig_extra.txt",join("\n",@chkextra),scalar @chkextra));
	$html .= htmltag("td","align=right",writeFile("byHost/".$h."_chkconfig_missing.txt",join("\n",@chkmissing),scalar @chkmissing));
	$html .= "</tr>\n";
      }
  }

$html .= "</table>\n";

$html .= "</html>\n";

my $out = "$confWeb/config.html";
open (OUT,">$out") || die "Cannot write to $out: $!";
print OUT $html;
close OUT;

exit 0;

sub statusColour
  {
    my $status = shift @_;
    return "bgcolor=red"    if $status > 1;
    return "bgcolor=yellow" if $status > 0;
    return "";
  }

sub findXmlTag
  {
    my $data = shift @_;
    my $name = shift @_;
    $data =~ /<$name>(.*)<\/$name>/s;
    return $1;
  }

sub SaveArr
  {
    my $file = shift @_;
    my $dataref = shift @_;
    $dataref = [] if ! defined $dataref;
    open (TMPOUT,">$confWeb/$file");
    print TMPOUT join("\n",@$dataref);
    close TMPOUT;
    return "<a href=\"$file\">".(scalar @$dataref)."</a>";
  }

sub htmltag
  {
    my $tag    = shift @_;
    my $params = shift @_;
    my @contents = @_;
    @contents = () unless defined $contents[0];
    my $content = join("",@contents);
    return "<$tag $params>$content</$tag>";
  }

sub readFile
  {
    my $filename = shift @_;
    #warn "Cannot read file $filename: $!" if ! -r $filename;
    #my $data = `cat $filename`;
    #return $data;
    if (open (TMPIN,$filename))
      {
	my $data = join("",<TMPIN>);
	close TMPIN;
	return $data;
      }
    else
      {
	warn "Cannot read file $filename: $!";
	return "";
      }
  }

sub writeFile
  {
    my $filename = shift @_;
    my $data     = shift @_;
    my $label    = shift @_;
    my $file     = "$confWeb/$filename";
    open (TMPOUT,">$file") || die "Cannot write $file: $!";
    print TMPOUT $data;
    close TMPOUT;
    #print STDERR "Write file $filename\n";
    if (length($data)<1)
      {
	return $label;
      }
    else
      {
	return htmltag("a","href=\"$filename\"",$label);
      }
  }

sub makeref
  {
    return \@_;
  }


sub showAll
  {
    my $allhosts = shift @_;
    my $reqhosts = shift @_;
    my $installeditems = shift @_;
    my $requireditems  = shift @_;
    my $data = shift @_;
    my %dupe;

    my $requireditemshash = listtohash(@{$requireditems});

    my $outall = "";
    my $outallcount = 0;
    my $outmissing = "";
    my $outmissingcount = 0;
    my $outextra = "";
    my $outextracount = 0;

    foreach my $item (sort @{$installeditems})
      {
	next if defined $dupe{$item};

	my %dupehost;

	foreach my $h (@{$$data{$item}{"hosts"}})
	  {
	    next if (! grep(/$h/,@{$allhosts}));
	    next if defined $dupehost{$h};
	    $dupehost{$h} = 1;
	  }

	$dupe{$item} = \%dupehost;

	my @xhosts = keys %dupehost;

	$outall .= $item .": ".join(", ",@xhosts)."\n";
	$outallcount++;

	if (defined $$requireditemshash{$item})
	  {
	  }
	else
	  {
	    $outextra .= $item .": ".join(", ",@xhosts)."\n";
	    $outextracount++;
	  }
      }

    foreach my $item (sort @{$requireditems})
      {
	my @missing;
	my $xhosts = $dupe{$item};

	#print "item=$item, xhosts=",$xhosts,"\n";

	foreach my $rh (sort @{$reqhosts})
	  {
	    push @missing,$rh if ! defined $$xhosts{$rh};
	  }
	
	if (@missing > 0)
	  {
	    $outmissing .= $item ." not on: ".join(", ",@missing)."\n";
	    $outmissingcount++;
	  }

	#die "Here!";
      }

    return ($outall,$outallcount,$outmissing,$outmissingcount,$outextra,$outextracount);
  }

sub listtohash
  {
    my %xhash;
    foreach my $v (@_)
      {
	$xhash{$v} = 1;
      }
    return \%xhash;
  }

sub splitRpm
  {
    my $rpm = shift @_;
    $rpm =~ /(.*)-(.*-.*)/;
    my $base    = $1;
    my $version = $2;
    return ($base, $version);
  }

#sub readRpms
#  {
#    my $file = shift @_;
#    my %rpms;
#    foreach my $rpm (sort split(/\n/,$file))
#      {
#	my ($rpmbase,$rpmversion) = splitRpm($rpm);
#	#print "Package: $rpm: $rpmbase, $rpmversion\n";
#	push @{$rpms{$rpmbase}},$rpmversion;
#      }
#    return \%rpms;
#  }

sub vercmp
  {
    my @ver1 = split(/[-\.]/,shift @_);
    my @ver2 = split(/[-\.]/,shift @_);

    #print "ver1: ".join("=",@ver1)."\n";
    #print "ver2: ".join("=",@ver2)."\n";

    for(my $i=0; ; $i++)
      {
	return -1 if !defined $ver1[$i];
	return +1 if !defined $ver2[$i];
	if (($ver1[$i]=~/^\d+$/)&&($ver2[$i]=~/^\d+$/))
	  {
	    my $cmp = $ver1[$i] <=> $ver2[$i];
	    #print "$i $cmp\n";
	    return $cmp if $cmp != 0;
	  }
	else
	  {
	    my $cmp = $ver1[$i] cmp $ver2[$i];
	    #print "$ver1[$i] $ver2[$i] $i $cmp\n";
	    return $cmp if $cmp != 0;
	  }
      }

    die "Error in vercmp!";
  }

#end file
